;+
;************************************************************************
;
; Copyright (C) 2021 The University Centre in Svalbard (UNIS)
; Author: Emma Bland (UNIS)
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Modifications:
;
;
;************************************************************************
;
; PURPOSE: Determine the number of clustered and isolated points in
;          fitted SuperDARN data, before and after despecking with
;          fit_speck_removal. Display results on a bar graph.
;
;
; Isolated points are identified using the same criteria as
; fit_speck_removal (i.e. less than 4 "neighbours" in range-time space).
;
; Before running this program, the rawacf files need to be processed
; using the accompanying bash script. You will also need to edit a few
; variables as indicated below.
; 
; This code requires the following packages:
;   - RST4.6
;   - IDL Coyote library (http://www.idlcoyote.com/documents/programs.php#COYOTE_LIBRARY_DOWNLOAD)
;     (used only for selecting plot colours)
;
; The data will be saved in an IDL sav file in the working directory so that you can make any 
; changes to the plot without having to read the data again. If you need to regenerate this 
; file (e.g. if radarlist has been modified), set makesav=1.
; 
;-
pro despeck_evaluation

;*************** EDIT HERE ********************

;path to the fitacf files
wkdir='/path/to/fitacf/files'

;date in YYYYMMDD format
daystmp='20160901'

;LIST OF RADARS (define your own list if desired. radars without data are plotted as an empty )
radarlist=['ade', 'adw', 'bks', 'bpk', 'cly', 'cve', 'cvw', 'dce', 'dcn', 'fhe', 'fhw', 'fir', 'gbr',$
           'hal', 'han', 'hkw', 'hok', 'inv', 'jme', 'kap', 'ker', 'kod', 'ksr', 'lyr', 'mcm', 'pgr',$
           'pyk', 'rkn', 'san', 'sas', 'sps', 'sto', 'sye', 'sys', 'tig', 'unw', 'wal', 'zho'] ;all 38 radars
;radarlist=['ade', 'adw', 'bks', 'bpk', 'cly'];custom list

;set this flag to 1 to re-generate the IDL SAV file
makesav=0

;**********************************************



nrad=n_elements(radarlist)

savfile=wkdir+'/'+daystmp+'.sav'

if makesav eq 1 or file_search(savfile) eq '' then begin

  ;SET UP ARRAYS TO STORE RESULTS
  valid_fits_v2=lonarr(nrad)
  valid_fits_v3=lonarr(nrad)
  valid_fits_v2_after_despeck=lonarr(nrad)
  valid_fits_v3_after_despeck=lonarr(nrad)
  valid_fits_v2_after_despeck_twice=lonarr(nrad)
  valid_fits_v3_after_despeck_twice=lonarr(nrad)
  nfiles=intarr(nrad)

  for irad=0,nrad-1 do begin

    radar=radarlist[irad]

    filestmp=wkdir+'/'+daystmp+'*'+radar+'*'
    fitacf2_files=file_search(filestmp+'.fitacf2',count=count)
    fitacf3_files=file_search(filestmp+'.fitacf3')
    fitacf2_despeck1_files=file_search(filestmp+'.fitacf2.despeck1')
    fitacf3_despeck1_files=file_search(filestmp+'.fitacf3.despeck1')
    fitacf2_despeck2_files=file_search(filestmp+'.fitacf2.despeck2')
    fitacf3_despeck2_files=file_search(filestmp+'.fitacf3.despeck2')
    nfiles[irad]=count
    if count eq 0 then continue

    for ifile=0,nfiles[irad]-1 do begin

      print,'Reading '+strmid(fitacf2_files[ifile],0,strlen(fitacf2_files[ifile])-7)+'*'
      fp_v2=fitopen(fitacf2_files[ifile])
      fp_v3=fitopen(fitacf3_files[ifile])
      fp_v2_despeck1=fitopen(fitacf2_despeck1_files[ifile])
      fp_v3_despeck1=fitopen(fitacf3_despeck1_files[ifile])
      fp_v2_depspeck2=fitopen(fitacf2_despeck2_files[ifile])
      fp_v3_depspeck2=fitopen(fitacf3_despeck2_files[ifile])

      s=0
      while s ne -1 do begin

        s=fitread(fp_v2,prm_v2,fit_v2)
        s=fitread(fp_v3,prm_v3,fit_v3)
        s=fitread(fp_v2_despeck1,prm_v2_despeck1,fit_v2_despeck1)
        s=fitread(fp_v3_despeck1,prm_v3_despeck1,fit_v3_despeck1)
        s=fitread(fp_v2_depspeck2,prm_v2_despeck2,fit_v2_despeck2)
        s=fitread(fp_v3_depspeck2,prm_v3_despeck2,fit_v3_despeck2)

        ;VALID FITS BEFORE DESPECKING
        valid_fits_v2[irad]=valid_fits_v2[irad]+total(fit_v2.qflg)
        valid_fits_v3[irad]=valid_fits_v3[irad]+total(fit_v3.qflg)

        ;VALID FITS AFTER DESPECKING
        valid_fits_v2_after_despeck[irad]=valid_fits_v2_after_despeck[irad]+total(fit_v2_despeck1.qflg)
        valid_fits_v3_after_despeck[irad]=valid_fits_v3_after_despeck[irad]+total(fit_v3_despeck1.qflg)

        ;VALID FITS AFTER DESPECKING TWICE
        ;  (used to calculate number of clustered points AFTER despecking)
        valid_fits_v2_after_despeck_twice[irad]=valid_fits_v2_after_despeck_twice[irad]+total(fit_v2_despeck2.qflg)
        valid_fits_v3_after_despeck_twice[irad]=valid_fits_v3_after_despeck_twice[irad]+total(fit_v3_despeck2.qflg)

      endwhile
      s=fitclose(fp_v2)
      s=fitclose(fp_v3)
      s=fitclose(fp_v2_despeck1)
      s=fitclose(fp_v3_despeck1)
      s=fitclose(fp_v2_depspeck2)
      s=fitclose(fp_v3_depspeck2)
    endfor

  endfor

  save,/variables,filename=savfile
endif else restore,savfile



;***************************************
;GENERATE PLOT
;***************************************
set_plot,'PS'
!P.font=0
!P.multi=[0,1,2]
!Y.OMargin=[1,12]
fig=wkdir+'/'+daystmp+'_despeck_statistics.eps'
device,filename=fig,/encapsulated,xsize=12+2.5*max([nrad,6]),ysize=45,/color 
colors=['CG1','CG3','CG5','CG4','CG11','CG9','ORG3','ORG2']
xoffset=0.17



for ipanel=0,1 do begin

  case ipanel of
    0:begin
      ytitle='Total number of valid fits'
      ymax=max(1.005*[valid_fits_v2,valid_fits_v3])
    end
    1:begin
      ytitle='Percentage of valid fits that are !Cclustered in range-time space'
      ymax=100
    end
  endcase

  plot,/nodata,[-4*xoffset,nrad-1+4*xoffset],[0,ymax],xstyle=1,xtickname=strupcase(radarlist),$
       xminor=1,xtickinterval=1,yticklen=0.005,charsize=2,ytitle=ytitle

  for irad=0,nrad-1 do begin

    if nfiles[irad] eq 0 then continue

    case ipanel of
      0:begin;to plot total number clustered & isolated points
        scaleFactor1=1.
        scaleFactor2=1.
        scaleFactor3=1.
        scaleFactor4=1.
      end
      1:begin;to plot percentage of clustered points
        scaleFactor1=100./float(max(valid_fits_v2[irad]))
        scaleFactor2=100./float(max(valid_fits_v2_after_despeck[irad]))
        scaleFactor3=100./float(max(valid_fits_v3[irad]))
        scaleFactor4=100./float(max(valid_fits_v3_after_despeck[irad]))
      end
    endcase


    ;FITACF2.5
    if scaleFactor1 ne 0 and finite(scaleFactor1) then begin
      x0=irad-2.25*xoffset & x1=x0+xoffset
      y0=0 & y1=valid_fits_v2[irad]*scalefactor1 & y2=valid_fits_v2_after_despeck[irad]*scalefactor1
      if ipanel eq 0 then polyfill,[x0,x1,x1,x0],[y0,y0,y1,y1],color=cgcolor(colors[1])
      polyfill,[x0,x1,x1,x0],[y0,y0,y2,y2],color=cgcolor(colors[0])
    endif

    ;FITACF2.5 (after despeck)
    if scaleFactor2 ne 0 and finite(scaleFactor2) then begin
      x0=irad-2.25*xoffset+1*xoffset & x1=x0+xoffset
      y0=0 & y1=valid_fits_v2_after_despeck[irad]*scalefactor2 & y2=valid_fits_v2_after_despeck_twice[irad]*scalefactor2
      if ipanel eq 0 then polyfill,[x0,x1,x1,x0],[y0,y0,y1,y1],color=cgcolor(colors[3])
      polyfill,[x0,x1,x1,x0],[y0,y0,y2,y2],color=cgcolor(colors[2])
    endif

    ;FITACF3.0
    if scaleFactor3 ne 0 and finite(scaleFactor3) then begin
      x0=irad-2.25*xoffset+2*xoffset+0.5*xoffset & x1=x0+xoffset
      y0=0 & y1=valid_fits_v3[irad]*scalefactor3 & y2=valid_fits_v3_after_despeck[irad]*scalefactor3
      if ipanel eq 0 then polyfill,[x0,x1,x1,x0],[y0,y0,y1,y1],color=cgcolor(colors[5])
      polyfill,[x0,x1,x1,x0],[y0,y0,y2,y2],color=cgcolor(colors[4])
    endif

    ;FITACF3.0 (after despeck)
    if scaleFactor4 ne 0 and finite(scaleFactor4) then begin
      x0=irad-2.25*xoffset+3*xoffset+0.5*xoffset & x1=x0+xoffset
      y0=0 & y1=valid_fits_v3_after_despeck[irad]*scalefactor4 & y2=valid_fits_v3_after_despeck_twice[irad]*scalefactor4
      if ipanel eq 0 then polyfill,[x0,x1,x1,x0],[y0,y0,y1,y1],color=cgcolor(colors[7])
      polyfill,[x0,x1,x1,x0],[y0,y0,y2,y2],color=cgcolor(colors[6])
    endif

  endfor

endfor



;MAKE LEGEND & ANNOTATE PLOT
items=['FitACF 2.5 - before despecking',$
       'FitACF 2.5 - after despecking',$
       'FitACF 3.0 - before despecking',$
       'FitACF 3.0 - after despecking']

UserSym, 4*[-1,1,1,-1,-1]+4, [1,1,-1,-1,1]+0.5, /Fill

!P.charsize=2
device_coords=convert_coord(!X.window[0],0,/normal,/to_device)
xpos1=device_coords[0] & xpos2=xpos1+4600 & ypos=0.9*!D.y_size & yoffset=0.0135*!D.y_size

xyouts,xpos1,ypos+2.3*yoffset,'   Clustered',/device
for icolor=0,n_elements(colors)-1,2 do begin
  plots,xpos1,ypos-yoffset*icolor,color=cgcolor(colors[icolor]),psym=8,symsize=5,/device,noclip=1
endfor

xyouts,xpos2,ypos+2.3*yoffset,'    Isolated',/device
for icolor=1,n_elements(colors)-1,2 do begin
  plots,xpos2,ypos-yoffset*(icolor-1),color=cgcolor(colors[icolor]),psym=8,symsize=5,/device,noclip=1
  xyouts,xpos2+4600,ypos-yoffset*(icolor-1),items[(icolor-1)/2],/device
endfor


;Add date stamp
xyouts,!X.window[0],0.97,/normal,strmid(daystmp,0,4)+'/'+strmid(daystmp,4,2)+'/'+strmid(daystmp,6,2)+'  '


device,/close_file
!P.multi=0
spawn,'evince '+fig+' &'


end
